/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas
**
** Classes for handling socket addresses.
**
*****************************************************************************/

#ifndef FANCYSOCK_ADDRESS_H
#define FANCYSOCK_ADDRESS_H

#include <fancyarray.h>
#include <fancysock.h>
#include <fancyobject.h>

#include <fancysockresolveoptions.h>


#ifdef WIN32
#define AF_LOCAL 99
extern "C" {
	struct sockaddr_un
	{
		short sun_family;
		char  sun_path[108];
	};
}
#endif

extern "C" {
  struct sockaddr;
}

namespace Fancy {

class String;

  namespace Sock {

  class IP6Address;

  /**
   * An abstract base class, which encapsulates the polymorphic
   * POSIX \c struct \c sockaddr.
   */
  class FANCYSOCK_API Address : virtual public Fancy::Object
  {
  protected:
	  Address() {}
	  virtual ~Address();

  public:
	  typedef Fancy::ObjPtr<Address> ptr_t;

	  enum AddressType {
		  UnixSocketAddress = 0,
		  IPv4Address = 1,
		  IPv6Address = 2
	  };

	  virtual AddressType getAddressType() const = 0;

	  /**
	   *
	   * Pure virtual function, which returns the pointer to the \c sockaddr
	   * structure of any derived class for usage in any POSIX socket function.
	   */
	  virtual const struct sockaddr *sockAddr() const = 0;

	  /**
	   * Pure virtual function, which returns the size of the \c sockaddr
	   * structure of any derived class for usage in any POSIX socket function.
	   */
	  virtual uint32_t sockLen() const = 0;

	  /**
	   * Return a string, which contains no port number for IP addresses.
	   * For local socket addresses, this function returns the same
	   * result than toString().
	   */
	  virtual std::string getHostString() const = 0;

	  /**
	   * Pure virtual function, which returns the domain of a socket corresponding
	   * to an address, as required for the first argument to socket(2).
	   */
	  virtual int addrDomain() const = 0;

	  /**
	   * Returns the port.
	   */
	  virtual uint16_t getPort() const = 0;

	  /**
	   * Return the original hostname, if this address originates from
	   * a call to @c resolve().
	   *
	   * If this is a local socket address or a plain IP address, this call
	   * return null.
	   *
	   * @since 3.5.0
	   */
	  virtual Fancy::ObjPtr<Fancy::String> getHostName() const = 0;

	  /**
	   * Returns a new instance of a derived Address class initialized from
	   * a given \c struct \c sockaddr and a \c socklen, as returned e.g.
	   * by accept().
	   *
	   * Throw a Fancysock::Exception, if the Address type is unknown.
	   */
	  static ptr_t newInstance(const struct sockaddr *addr, uint32_t len);

	  /**
	   * Returns a new instance of a derived Address class initialized from
	   * a given String.
	   *
	   * Currently supported formats are:
	   * - local sockets in the form of an absolute path '/xxx/yyy/...'
	   * - the IPv4 Format 'ddd.ddd.ddd.ddd' and 'ddd.ddd.ddd.ddd:port'
	   * - the IPv6 Format like '::1' and '[::1]:port'
	   * Throws an Exepction, if the String is invalid.
	   */
	  static ptr_t newInstance(const std::string &address, uint16_t default_port=0);

	  /**
	   * Returns a new instance of a derived Address class initialized from
	   * a given String, using the specified hostname instead of resolving it.
	   *
	   * Currently supported formats are:
	   * - local sockets in the form of an absolute path '/xxx/yyy/...'
	   * - the IPv4 Format 'ddd.ddd.ddd.ddd' and 'ddd.ddd.ddd.ddd:port'
	   * - the IPv6 Format like '::1' and '[::1]:port'
	   * Throws an Exepction, if the String is invalid.
	   */
	  static ptr_t newInstance(const Fancy::ObjPtr<Fancy::String> &hostname, const std::string &address, uint16_t default_port=0);

	  /**
	   *  Returns a new instance of a derived Address class initialized from
	   *  a given String.
	   *
	   * Currently supported formats are:
	   * - local sockets in the form of an absolute path '/xxx/yyy/...'
	   * - the IPv4 Format 'ddd.ddd.ddd.ddd' and 'ddd.ddd.ddd.ddd:port'
	   * - the IPv6 Format like '::1' and '[::1]:port'
	   * - 'hostname' and 'hostname:port', in which case a
	   *   nameserver is contacted in order to resolve the address to an IPv4 address.
	   *   tryV6, preferV6, v6only only apply if address does not look like a local, v4 or v6 address.
	   * Throws an Exepction, if the String is invalid.
	   *
       * @param resolveOptions defaults to ResolveOptions::PreferIPv4
	   */
	  static ptr_t resolve(const std::string &name, uint16_t default_port=0, const Fancy::ObjPtr<ResolveOptions>& resolveOptions=0);

	  /**
	   * Return a new instance, which represents a UNIX domain socket or
	   * a Windows pipe. The name must start with the string returned by
	   * @c getLocalPrefix().
	   */
	  static ptr_t newLocalInstance (const std::string &path);

	  /**
	   * Return a new instance, which represents the given IPv4 address.
	   *
	   * @param addr The IP address in host byte order.
	   *             The localhost address @c 127.0.0.1 is e.g. represented by the
	   *             value @c 0x7f000001.
	   * @param port The port to set to the returned address instance.
	   */
	  static ptr_t newIP4Instance (uint32_t addr, uint16_t port);

	  /**
	   * Return a new instance, which represents the given IPv6 address.
	   *
	   * @param addr The IP address in host byte order.
	   *             The localhost address @c ::1 is e.g. represented by the value
	   *             @c { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 }
	   * @param port The port to set to the returned address instance.
	   */
	  static Fancy::ObjPtr<IP6Address> newIP6Instance (Fancy::ObjPtr<Fancy::ArrayObject<uint8_t> > &addr, uint16_t port);

	  /**
	   * Return the operating system dependent prefix for local sockets
	   * resp. named pipes.
	   *
	   * @return @c "\\.\pipe\" under Windows or @c "/" under UNIX.
	   */
	  static std::string getLocalPrefix();

  };


  class FANCYSOCK_API IP6Address : public Address
  {

  protected:
  	virtual ~IP6Address();

  public:
  	typedef Fancy::ObjPtr<IP6Address> ptr_t;


  	virtual Address::AddressType getAddressType() const = 0;

  	virtual std::string toString() const = 0;
  	virtual bool lessThan(const Fancy::Object &o) const = 0;
  	virtual bool equals(const Fancy::Object &o) const = 0;

  	virtual const struct sockaddr *sockAddr() const = 0;
  	virtual uint32_t sockLen() const = 0;
  	virtual int addrDomain() const = 0;

  	/**
  	 * returns the addess in shortest form, like "::1", "2001:1234::4321"
  	 */
  	virtual std::string getHostString() const = 0;

  	// additional special functions

  	/**
  	 * returns the addess in half-shorted form without empty parts,
  	 * but with obsolete leading '0's ommited,
  	 * like "0:0:0:0:0:0:1", "2001:1234:0:0:0:0:0:4321"
  	 */
  	virtual std::string getNonEmptyHostString() const = 0;

  	/**
  	 * tests if the address is in 0:0:0:0:0:ffff::/96, like a mapped IPv4 client connected to a dual stack server
  	 */
    virtual bool isMappedV4Address() const = 0;

  	/**
  	 * if isMappedV4Address(), returns a new IP4Instance(). Exception thrown otherwise.
  	 */
    virtual Fancy::ObjPtr<Address> mappedV4Address() const = 0;

  };


  }
}

#endif // ! FANCYSOCK_ADDRESS_H
