/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKBEANIMPORTER_H_
#define FANCYSOCKBEANIMPORTER_H_

#include <fancysockcallbackinterfaceaware.h>
#include <fancysessioncontextfactory.h>

namespace Fancy {

class String;
class Map;

namespace Sock {

class RemotingClient;
class RemotingClientPool;

class FANCYSOCK_API BeanImporter: public virtual CallbackInterfaceAware,
                                  public virtual Fancy::SessionContextFactory {
protected:
    BeanImporter();
    virtual ~BeanImporter();
public:
    typedef Fancy::ObjPtr<BeanImporter> ptr_t;

    /**
     * Return a new instance with a default specific key.
     */
    static ptr_t newInstance();

    /**
     * Return a new instance with the specified specific key.
     */
    static ptr_t newInstance(const Fancy::Object::ptr_t &specificKey);

    /**
     * Return a new instance with a default specific key. The client instances are
     * pulled from the given client pool.
     */
    static ptr_t newInstance(const Fancy::ObjPtr<RemotingClientPool> &pool);

    /**
     * Return a new instance with the specified specific key. The client instances are
     * pulled from the given client pool.
     */
    static ptr_t newInstance(const Fancy::ObjPtr<RemotingClientPool> &pool, const Fancy::Object::ptr_t &specificKey);

    /**
     * @param The metadata passed to the remote invocation. If @c sessionContextSubSystem
     *        is not @c null, this metadata is paased ony to the open request of the session
     *        context.
     */
    virtual void setMetadata(const Fancy::ObjPtr<Map> &metadata) = 0;

    /**
     * @return The metadata passed to the remote invocation. If @c sessionContextSubSystem
     *        is not @c null, this metadata is paased ony to the open request of the session
     *        context.
     */
    virtual Fancy::ObjPtr<Map> getMetadata() const = 0;

    /**
     * @param subSystem The subsystem, which handles calls to returned callback proxies.
     */
    virtual void setReturnCallbackSubSystem(const Fancy::ObjPtr<Fancy::String> &subSystem) = 0;

    /**
     * @return The subsystem, which handles calls to returned callback proxies.
     */
    virtual Fancy::ObjPtr<Fancy::String> getReturnCallbackSubSystem() const = 0;

    /**
     * @param subSystem The subsystem, which handles requests to open a remote session context.
     *                  If set to @c null, no open and close session request is issued to
     *                  the server when acquiring a new session context.
     */
    virtual void setSessionContextSubSystem(const Fancy::ObjPtr<Fancy::String> &subSystem) = 0;

    /**
     * @return The subsystem, which handles requests to open a remote session context.
     *                  If set to @c null, no open and close session request is issued to
     *                  the server when acquiring a new session context.
     */
    virtual Fancy::ObjPtr<Fancy::String> getSessionContextSubSystem() const = 0;

   /**
     * @param The client which is used to establish the session.
     * @return A new session context. The context is bound to the current thread using the
     *         specific key returned by @c getSpecificKey().
     *         If the returned object is destroyed, the remote session context is closed
     *         and the local context is unbound from the current thread.
     */
    virtual Fancy::Object::ptr_t newClientSessionContext(const Fancy::ObjPtr<RemotingClient> &client) = 0;

    /**
     * Generate a proxy, which issues call to the given remote subsystem and represents the type of the
     * given reflection target.
     *
     * The returned proxy takes the client bound to the current thread using @c newSessionContext()
     * and should be used as a singleton, because constructing a proxy using this function is
     * inherently slow, because the while interface has to be inspected.
     *
     * @param subSystem The subsystem to which the remote call is directed.
     * @param A reflection target, which is usually retrieved through the static function
     *        @c Bean::reflect(null) and is used in order to retrieve type informations like
     *        method names, method parameter types and return types of methods.
     */
    virtual Fancy::ObjPtr<Fancy::ReflectionTarget> newProxy(const Fancy::ObjPtr<Fancy::String> &subSystem,
                                                            const Fancy::ObjPtr<Fancy::ReflectionTarget> &typeInfo) = 0;

    /**
     * Generate a local proxy for a remote invocation handle, which implements the given bean class.
     *
     * The returned proxy takes the client bound to the current thread using @c newSessionContext().
     * and should be used as a singleton, because constructing a proxy using this function is
     * inherently slow, because the while interface has to be inspected.
     *
     * @param Bean The type of the bean to generate a proxy for.
     * @param subSystem The subsystem to which the remote call is directed.
     */
    template <class Bean>
    Fancy::ObjPtr<Bean> newBeanProxy(const Fancy::ObjPtr<Fancy::String> &subSystem) {
        return Bean::newProxyInstance(this->newProxy(subSystem,Bean::reflect(Fancy::ObjPtr<Bean>())));
    }
};

}

}

#endif /* FANCYSOCKBEANIMPORTER_H_ */
