/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / iteg
**
** A base class, which reads and writes document with MIME type
** multipart/mixed
**
*****************************************************************************/

#ifndef FANCYSOCK_MULTIPART_TRANSPORT_H
#define FANCYSOCK_MULTIPART_TRANSPORT_H

#include <fancysocktransport.h>

namespace Fancy {

namespace Sock {

/**
  An implementation of the multipart MIME type as of RFC2046

  https://tools.ietf.org/html/rfc2046

  Reading is done by calling

  bdry = MultipartTransport::getMultipartBoundary(headers);
  if (!bdry.isNull()) {

    MultipartTransport::ptr_t mp = MultipartTransport::newInstance(channel,bdry);

    Map::ptr_t headers;

    while (!(header=mp->openReadDoc()).isNull()) {

        // read from mp until mp->read() returns 0...
    }
  }

  Writing is performed by

    bdry = generateRandomBoundary();

    MultipartTransport::ptr_t mp = MultipartTransport::newInstance(channel,bdry);

    forall (parts) {
        mp->openWriteDoc(header);

        // write to mp
    }

    mp->flush();
 */
class FANCYSOCK_API MultipartTransport : virtual public Transport
{
protected:
    MultipartTransport();

    virtual ~MultipartTransport();

public:
    typedef Fancy::ObjPtr<MultipartTransport> ptr_t;

    /**
     *  Constructs a multipart transport using the given channel and the
     *  given buffer size.
     *
     *  If you want to use the returned instance for reading,
     *  pass a boundary as returned by @c getMultipartBoundary()
     *  to this method.
     *
     *  If you want to use the returned instance for writing,
     *  pass a boundary as generated by generateBoundary().
     */
    static ptr_t newInstance(const Fancy::ObjPtr<Channel> &d,
            const Fancy::ObjPtr<Fancy::String> &boundary,
            int bufsize = 16384);

    /**
     * Generate a random boundary of length 20 in the form
     *
     *   "----lksdjsdfsgkjhlaf"
     *
     * where the last 16 characters are built out of 32 characters.
     */
    static Fancy::ObjPtr<Fancy::String> generateRandomBoundary();

    /**
     * Return the multipart boundary of the given value of
     * a Content-Type headers contained in the given headers map,
     * which must conform to the syntax
     *
     * multipart/mixed; boundary=gc0p4Jq0M2Yt08j34c0p
     *
     * Returns null, if no content-type header is contained in the given map
     * or the header does not conform to this syntax.
     */
    static Fancy::ObjPtr<Fancy::String> getMultipartBoundary(const Fancy::ObjPtr<Fancy::Map> &headers);

    /**
     * Return the multipart boundary of the given value of
     * a Content-Type headers in the form
     *
     * multipart/mixed; boundary=gc0p4Jq0M2Yt08j34c0p
     *
     * Returns null, if the header does not conform to this syntax.
     */
    static Fancy::ObjPtr<Fancy::String> getMultipartBoundary(const std::string &contentType);

    /**
     * Set the multipart boundary may be set between individual parts,
     * i.e. after calling flush() or after a null return value
     * from openWriteDoc().
     */
    virtual void setBoundary(const Fancy::ObjPtr<Fancy::String> &bdry) = 0;

    /**
     * Get the multipart boundary, which is currently used.
     */
    virtual Fancy::ObjPtr<Fancy::String> getBoundary() const = 0;

private:
    MultipartTransport (const MultipartTransport &);
    MultipartTransport &operator=(const MultipartTransport &);
};

}}

#endif // ! FANCYSOCK_MULTIPART_TRANSPORT_H
