/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A base class, which wraps TCP/IP sockets for use with libxml.
** Extensible for using SSL or socks etc. by using many virtual functions.
**
*****************************************************************************/

#ifndef FANCYSOCK_PACKET_DEVICE_H
#define FANCYSOCK_PACKET_DEVICE_H

#include <fancypollablechannel.h>
#include <fancysockaddress.h>

namespace Fancy {
namespace Sock {

/** A base class for packet sockets.

    This class defines an socket device, which is represented by
    a POSIX filedecriptor as returned by socket(2).

    It may be reimplemented in order to use many other socket encapsulations
    such as SSL or SOCKS.
  */
  class FANCYSOCK_API PacketDevice : virtual public Fancy::PollableChannel
  {
  public:
	  typedef Fancy::ObjPtr<PacketDevice> ptr_t;

	  /**
	   * Instantiate a new ICMP instance for IPv4 use.
       * This legacy method is equivalent to ICMPInstanceForFamiliy(AF_INET)
       *
       * This method will use close_on_exec=true
	   */
	  static ptr_t newICMPInstance();

      /**
       * Instantiate a new ICMP instance for IPv4 use.
       * This legacy method is equivalent to ICMPInstanceForFamiliy(AF_INET,close_on_exec)
       *
       * @param close_on_exec Whether the socket will be closed when any of the
       *                      exec*() functions is used to run another program,
       *                      e.g. in a forked instance of this process.
       *                      This flag is ignored under Windows.
       */
      static ptr_t newICMPInstance(bool close_on_exec);

      /**
       * Instantiate a new ICMP instance for IPv4 or IPv6 use.
       *
       * This method will use close_on_exec=true
       *
       * @param addressFamily An address family, AF_*, as returned by Address::addrDomain()
       */
      static ptr_t newICMPInstanceForFamiliy(int addressFamily);

      /**
       * Instantiate a new ICMP instance for IPv4 or IPv6 use.
       *
       * @param addressFamily An address family, AF_*,  as returned by Address::addrDomain()
       * @param close_on_exec Whether the socket will be closed when any of the
       *                      exec*() functions is used to run another program,
       *                      e.g. in a forked instance of this process.
       *                      This flag is ignored under Windows.
       */
      static ptr_t newICMPInstanceForFamiliy(int addressFamily, bool close_on_exec);

      /**
       * Instantiate a new UDP instance for IPv4 client use.
       * This legacy method is equivalent to newDGRAMInstance(AF_INET)
       *
       * This method will use close_on_exec=true
       */
      static ptr_t newUDPInstance();

      /**
       * Instantiate a new UDP instance for IPv4 client use.
       * This legacy method is equivalent to newDGRAMInstance(AF_INET,close_on_exec)
       *
       * @param close_on_exec Whether the socket will be closed when any of the
       *                      exec*() functions is used to run another program,
       *                      e.g. in a forked instance of this process.
       *                      This flag is ignored under Windows.
       */
      static ptr_t newUDPInstance(bool close_on_exec);

      /**
       * Instantiate a new unconnected and unbound client datagram instance
       * for the given address family as returned by Address::addrDomain()
       *
       * Under UNIX and windows, a UDP socket is returned for AF_INET and AF_INET6.
       *
       * Under UNIX only, AF_LOCAL is recognized in order to use a UNIX domain socket
       * in datagram mode.
       *
       * @param addressFamily An address family, AF_*
       * @param close_on_exec Whether the socket will be closed when any of the
       *                      exec*() functions is used to run another program,
       *                      e.g. in a forked instance of this process.
       *                      This flag is ignored under Windows.
       */
      static ptr_t newDGRAMInstance(int addressFamily, bool close_on_exec);

      /**
       * Instantiate a new unconnected and unbound client datagram instance
       * for the given address family as returned by Address::addrDomain()
       *
       * Under UNIX and windows, a UDP socket is returned for AF_INET and AF_INET6.
       *
       * Under UNIX only, AF_LOCAL is recognized in order to use a UNIX domain socket
       * in datagram mode.
       *
       * This method will use close_on_exec=true
       *
       * @param addressFamily An address family, AF_*
       */
      static ptr_t newDGRAMInstance(int addressFamily);

      /**
       * Instantiate and bind a new UDP instance and bind it to the given address.
       *
       * This method will use close_on_exec=true
       *
       * Under UNIX, you may pass a LocalAddress instance, which will result in
       * binding to a UNIX domain socket.
       */
      static ptr_t bindUDP (const Address::ptr_t &addr);

      /**
       * Instantiate and bind a new UDP instance and bind it to the given address.
       *
       * @param close_on_exec Whether the socket will be closed when any of the
       *                      exec*() functions is used to run another program,
       *                      e.g. in a forked instance of this process.
       *                      This flag is ignored under Windows.
       */
      static ptr_t bindUDP (const Address::ptr_t &addr, bool close_on_exec);

     /**
	   * Receive a datagram and return the number of bytes received.
	   * The addr pointer is set to the origin of the datagram.
	   */
	  virtual int recvFrom(void *buf, int len, Address::ptr_t &addr) = 0;

	  /**
	   * Send a datagram to the given address.
	   */
	  virtual int sendTo(const void *buf, int len, const Address::ptr_t &addr) = 0;

	  /**
	   * Set the receive timeout of the socket in milliseconds as per the SO_RECVTIMEO
	   * socket option under UNIX.
	   */
	  virtual void setSoRecvTimeout(int ms) = 0;

	  /**
	   * Get the receive timeout of the socket in milliseconds as per the SO_RECVTIMEO
	   * socket option under UNIX.
	   */
	  virtual int getSoRecvTimeout() const = 0;

  protected:
	  PacketDevice();
	  virtual ~PacketDevice();
  };


  }
}

#endif // ! FANCYSOCK_PACKET_DEVICE_H
