/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** Highlevel socket server class for handling multiple connections
** in seperate threads.
**
*****************************************************************************/

#ifndef FANCYSOCK_SERVER_H
#define FANCYSOCK_SERVER_H

#include <string>
#include <iosfwd>
#include <fancythread.h>
#include <fancysockstateful.h>

namespace Fancy {

class ThreadPool;

namespace Sock {

// fwd decls.
class Address;
class Device;
class ConnectionHandlerFactory;

/**
  A Server is handling a socket bound to a specific address.

  The user of a server must supply a @c ConnectionHandlerFactory
  in order to create application-specific socket loops.

  The default implementation splits off as many connection handling threads
  as the configured maximal number of connections.
 */
class FANCYSOCK_API Server : virtual public Fancy::Runnable, virtual public Stateful
{
public:
	typedef Fancy::ObjPtr<Server> ptr_t;

	/**
	 * Instantiates a new default server implementation, which instantiates
	 * client connection handlers from the given ConnectionHandlerFactory.
	 */
	static ptr_t newInstance(const Fancy::ObjPtr<ConnectionHandlerFactory> &chf, const Fancy::ObjPtr<ThreadPool> &pool);

	/**
	 * Set a runnable, which is executed periodically from within the server's
	 * main loop.
	 */
	virtual void setGCRunnable(const Fancy::Runnable::ptr_t &runnable) = 0;

    /**
     * Sets the interval at which we check for closed client connections.
     * The default interval is @c 500ms.
     */
	virtual void setGCInterval(int msec) = 0;

	/**
	 * @return The runnable, which is executed periodically from within the server's
	 * main loop.
	 */
	virtual Fancy::Runnable::ptr_t getGCRunnable() const = 0;

    /**
      @return the interval at which we check for closed client connections.
    */
 	virtual int getGCInterval() const = 0;

    /*!
      Associates this server the given bound socket and allocates resources.
      It is assumed, that @c Device::listen() has been called on the given socket before.

      After calling this method, @c run() should be called in order to enter the main loop.
    */
	virtual void bind (const Fancy::ObjPtr<Device> &dev) = 0;

	// Runnable
	virtual void run() = 0;
	virtual void requestInterruption() = 0;

protected:
	Server ();
	virtual ~Server();

private:
	Server *prev;
	Server *next;
};

}
}

#endif // ! FANCYSOCK_SERVER_H
