/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2017 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKWSHTTPDREQUESTHANDLER_H_
#define FANCYSOCKWSHTTPDREQUESTHANDLER_H_

#include <fancysockhttpdrequesthandler.h>
#include <fancysockwstransport.h>


namespace Fancy {

class Collection;
class Exception;


namespace Sock {

class HttpServerTransport;


namespace HttpD {

class SessionManager;
class WsHttpDRequestHandler;


class FANCYSOCK_API WsHttpDRequestHandlerListener: public Fancy::Object {

protected:
  virtual ~WsHttpDRequestHandlerListener() {};

public:
  typedef ObjPtr<WsHttpDRequestHandlerListener> ptr_t ;

  /**
   * Upgrading the HTTP connection to a WebSocket connection failed
   */
  virtual void failed(WsHttpDRequestHandler *requestHandler,
                      const Fancy::ObjPtr<HttpServerTransport> &httpServerTransp,
                      const std::exception &e) = 0;

  /**
   * HTTP connection upgraded successfully to a WebSocket
   */
  virtual void connected(WsHttpDRequestHandler *requestHandler,
                         const Fancy::ObjPtr<WsTransport> &wsTransport) = 0;

  /**
   * WebSocket closed in a normal way.
   */
  virtual void closed(WsHttpDRequestHandler *requestHandler,
                      const Fancy::ObjPtr<WsTransport> &wsTransport) = 0;

  /**
   * WebSocket connection aborted
   */
  virtual void aborted(WsHttpDRequestHandler *requestHandler,
                       const Fancy::ObjPtr<WsTransport> &wsTransport, const std::exception &e) = 0;

  /**
   * New WebSocket frame received
   */
  virtual void frameReceived(WsHttpDRequestHandler *requestHandler,
                             const Fancy::ObjPtr<WsTransport> &wsTransport) = 0;
};


class FANCYSOCK_API WsHttpDRequestHandler: public virtual HttpDRequestHandler {

protected:
  Fancy::ObjPtr<WsHttpDRequestHandlerListener> listener;
  Fancy::ObjPtr<Collection> subProtocols;
  Fancy::ObjPtr<SessionManager> sessionManager;

  WsHttpDRequestHandler(const Fancy::ObjPtr<WsHttpDRequestHandlerListener> &listener,
                        const Fancy::ObjPtr<Collection> &subProtocols,
                        const Fancy::ObjPtr<SessionManager> &sessionManager);
  virtual ~WsHttpDRequestHandler();

  virtual void connected(const Fancy::ObjPtr<WsTransport> &wsTransp);
  virtual void disconnected(const Fancy::ObjPtr<WsTransport> &wsTransp);
  virtual void disconnected(const Fancy::ObjPtr<WsTransport> &wsTransp, const std::exception &e);
  virtual void frameReceived(const Fancy::ObjPtr<WsTransport> &wsTransp);

public:
  typedef Fancy::ObjPtr<WsHttpDRequestHandler> ptr_t;

  virtual void handleRequest(const Fancy::ObjPtr<Map> & requestHeaders,
                             const Fancy::ObjPtr<HttpServerTransport> & httpServerTransport);

  /**
   * Creates a new WebSocket request handler instance.
   *
   * @param listener A 'WsHttpDRequestHandlerListener' instance. Must not be Null!
   * @param subProtocols A collection of sub-protocol names, which are supported by the
   *                     WebSocket server.
   *
   * @return A pointer to a 'WsHttpDRequestHandler' instance
   */
  static ptr_t newInstance(const Fancy::ObjPtr<WsHttpDRequestHandlerListener> &listener,
                           const Fancy::ObjPtr<Collection> &subProtocols = NULL,
                           const Fancy::ObjPtr<SessionManager> &sessionManager = NULL);
};


class FANCYSOCK_API WsHttpDBroadcastRequestHandlerListener: public WsHttpDRequestHandlerListener {

protected:
  virtual ~WsHttpDBroadcastRequestHandlerListener() {};

public:
  typedef ObjPtr<WsHttpDBroadcastRequestHandlerListener> ptr_t ;

  /**
   * Notifies the actual implementation, that the request-handler requires exclusive
   * write-access to the given 'WsTransport' instance. This means, that the access to
   * any of the  WsTransport's 'write()' methods must be blocked, e.g. by using a
   * mutex, until the 'exclusiveWrEnd()' method gets called.
   */
  virtual void exclusiveWrBegin(WsHttpDRequestHandler *requestHandler,
                                const Fancy::ObjPtr<WsTransport> &wsTransport) = 0;

  /**
   * Notifies the actual implementation, that the request-handler does not longer require
   * an exclusive write-access to the given 'WsTransport' instance.
   */
  virtual void exclusiveWrEnd(WsHttpDRequestHandler *requestHandler,
                              const Fancy::ObjPtr<WsTransport> &wsTransport) = 0;
};


// client filter for WsHttpDBroadcastRequestHandler::multicast
class FANCYSOCK_API WsMulticastFilter : public virtual Fancy::Object {

protected:
  WsMulticastFilter();
  virtual ~WsMulticastFilter();

public:
  typedef Fancy::ObjPtr<WsMulticastFilter> ptr_t;

  virtual bool includeInRecipients(const Fancy::ObjPtr<WsTransport> & wsTransport) const = 0;
};


class FANCYSOCK_API WsHttpDBroadcastRequestHandler: public virtual WsHttpDRequestHandler {

protected:
  WsHttpDBroadcastRequestHandler(
    const Fancy::ObjPtr<WsHttpDBroadcastRequestHandlerListener> &listener,
    const Fancy::ObjPtr<Collection> &subProtocols,
    const Fancy::ObjPtr<SessionManager> &sessionManager);
  virtual ~WsHttpDBroadcastRequestHandler();

public:
  typedef Fancy::ObjPtr<WsHttpDBroadcastRequestHandler> ptr_t;

  /**
   * Returns the count of currently connected clients.
   */
  virtual size_t getConnectedCount() const = 0;

  /**
   * Broadcasts a single frame to all connected clients.
   *
   * @param  payload The payload data to be sent via the broadcast
   * @param  payloadLen The size of the payload data.
   *
   * @return The number of clients, which the broadcast was sent to.
   */
  virtual size_t broadcast(const void *payload, int payloadLen,
    WsTransport::DataType dataType = WsTransport::BinaryData) = 0;

  /**
   * Broadcasts a single frame to all connected clients.
   *
   * @param  payload The payload data to be sent via the broadcast
   * @param  payloadLen The size of the payload data.
   * @param filter The filter for wether to include a WsTransport in recipients
   *
   * @return The number of clients, which the broadcast was sent to.
   */
  virtual size_t multicast(const void *payload, int payloadLen,
    WsTransport::DataType dataType = WsTransport::BinaryData,
    const Fancy::ObjPtr<WsMulticastFilter> & filter = NULL) = 0;

  /**
   * Performs session timeout checks for all connections with WsTransport with session set.
   *
   */
  virtual void performSessionTimeoutChecks() = 0;

  /**
   * Creates a new WebSocket request handler instance, which also allows to broadcast a message
   * to all connected WebSocket clients.
   *
   * @param listener A 'WsHttpDBroadcastRequestHandlerListener' instance. Must not be Null!
   * @param subProtocols A collection of sub-protocol names, which are supported by the
   *                     WebSocket server.
   *
   * @return A pointer to a 'WsHttpDBroadcastRequestHandler' instance
   */
  static ptr_t newInstance(const Fancy::ObjPtr<WsHttpDBroadcastRequestHandlerListener> &listener,
                           const Fancy::ObjPtr<Collection> &subProtocols = NULL,
                           const Fancy::ObjPtr<SessionManager> &sessionManager = NULL);
};



}
}
}

#endif /* FANCYSOCKWSHTTPDREQUESTHANDLER_H_ */
